@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2020 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2022 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -111,9 +111,6 @@
 #define STATUSHEIGHT  2
 #define MAX_NOTESIZ   40
 
-/* Format for appointment hours is: HH:MM */
-#define HRMIN_SIZE 6
-
 /* Maximum number of colors available. */
 #define NBUSERCOLORS  6
 
@@ -155,7 +152,7 @@
  * The argument (d) is the "Sunday"-numbering of member tm_wday in struct tm.
  */
 #define WDAY(d) \
-    (ui_calendar_week_begins_on_monday() ? ((d ? d : WEEKINDAYS) - 1) : d)
+	(modify_wday(d, -ui_calendar_get_wday_start()))
 
 /* Key definitions. */
 #define CTRLVAL         0x1F
@@ -306,6 +303,7 @@ struct conf {
 	int input_datefmt;	/* format for reading date */
 	enum pos heading_pos; /* left/center/right for heading in appts panel */
 	char day_heading[BUFSIZ];	/* format for displaying heading in appts panel */
+	char timefmt[BUFSIZ];	/* format for displaying time in appts panel*/
 };
 
 #define EMPTY_DAY_DEFAULT "--"
@@ -338,6 +336,9 @@ enum datefmt {
 /* Day heading default format. */
 #define DAY_HEADING_DEFAULT "%B %e, %Y"
 
+/* Appointment time default format. */
+#define APPT_TIME_DEFAULT "%H:%M"
+
 /*
  * Calcurse representation of the date of a day in the calendar.
  * When time_t is a 32-bit signed integer, the year range is 1902 - 2037.
@@ -515,8 +516,8 @@ struct io_file {
 	char *name;
 };
 
-/* Available keys. */
-enum key {
+/* Virtual keys. */
+enum vkey {
 	KEY_GENERIC_CANCEL,
 	KEY_GENERIC_SELECT,
 	KEY_GENERIC_CREDITS,
@@ -527,6 +528,7 @@ enum key {
 	KEY_GENERIC_COPY,
 	KEY_GENERIC_PASTE,
 	KEY_GENERIC_CHANGE_VIEW,
+	KEY_GENERIC_PREV_VIEW,
 	KEY_GENERIC_IMPORT,
 	KEY_GENERIC_EXPORT,
 	KEY_GENERIC_GOTO,
@@ -566,7 +568,7 @@ enum key {
 	KEY_RAISE_PRIORITY,
 	KEY_LOWER_PRIORITY,
 
-	NBKEYS,
+	NBVKEYS,
 	KEY_UNDEF,
 
 	/* Non-configurable, context sensitive key bindings. */
@@ -809,7 +811,7 @@ void ui_calendar_set_current_date(void);
 struct date *ui_calendar_get_today(void);
 void ui_calendar_set_first_day_of_week(enum wday);
 void ui_calendar_change_first_day_of_week(void);
-unsigned ui_calendar_week_begins_on_monday(void);
+int ui_calendar_get_wday_start(void);
 void ui_calendar_store_current_date(struct date *);
 void ui_calendar_init_slctd_day(void);
 struct date *ui_calendar_get_slctd_day(void);
@@ -855,6 +857,7 @@ void day_display_item_date(struct day_item *, WINDOW *, int, time_t, int, int);
 void day_display_item(struct day_item *, WINDOW *, int, int, int, int);
 void day_write_stdout(time_t, const char *, const char *, const char *,
 		      const char *, int *);
+void day_do_storage(int day_changed);
 void day_popup_item(struct day_item *);
 int day_check_if_item(struct date);
 unsigned day_chk_busy_slices(struct date, int, int *);
@@ -946,24 +949,26 @@ int io_get_modified(void);
 void keys_init(void);
 void keys_free(void);
 void keys_dump_defaults(char *);
-const char *keys_get_label(enum key);
-enum key keys_get_action(int);
+const char *keys_get_label(enum vkey);
+const char *keys_get_binding(enum vkey);
+enum vkey keys_get_action(int);
 int keys_wgetch(WINDOW *);
 void keys_wait_for_any_key(WINDOW *);
-enum key keys_get(WINDOW *, int *, int *);
-int keys_assign_binding(int, enum key);
-void keys_remove_binding(int, enum key);
+enum vkey keys_get(WINDOW * win, int *, int *);
+int keys_assign_binding(int, enum vkey);
+void keys_remove_binding(int, enum vkey);
 int keys_str2int(const char *);
 char *keys_int2str(int);
-int keys_action_count_keys(enum key);
-const char *keys_action_firstkey(enum key);
-const char *keys_action_nkey(enum key, int);
-char *keys_action_allkeys(enum key);
+int keys_action_count_keys(enum vkey);
+const char *keys_action_firstkey(enum vkey);
+const char *keys_action_nkey(enum vkey, int);
+char *keys_action_allkeys(enum vkey);
 void keys_display_bindings_bar(WINDOW *, int *, int, int, int);
-void keys_popup_info(enum key);
+void keys_popup_info(enum vkey);
 void keys_save_bindings(FILE *);
-int keys_check_missing_bindings(void);
-void keys_fill_missing(void);
+int keys_check_missing(void);
+int keys_check_undefined(void);
+int keys_fill_missing(void);
 
 /* listbox.c */
 void listbox_init(struct listbox *, int, int, int, int, const char *,
@@ -1230,6 +1235,8 @@ time_t date_sec_change(time_t, int, int);
 time_t update_time_in_date(time_t, unsigned, unsigned);
 time_t get_sec_date(struct date);
 long min2sec(unsigned);
+int modify_wday(int,int);
+char *get_wday_default_string(int);
 void draw_scrollbar(struct scrollwin *, int);
 void item_in_popup(const char *, const char *, const char *, const char *);
 time_t get_today(void);
@@ -1325,6 +1332,7 @@ void wins_sbar_wdec(void);
 enum win wins_slctd(void);
 void wins_slctd_set(enum win);
 void wins_slctd_next(void);
+void wins_slctd_prev(void);
 void wins_init(void);
 void wins_scrollwin_init(struct scrollwin *, int, int, int, int, const char *);
 void wins_scrollwin_resize(struct scrollwin *, int, int, int, int);
