@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2020 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2022 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,7 @@
 #include <stdlib.h>
 #include <math.h>
 #include <ctype.h>
+#include <langinfo.h>
 
 #include "calcurse.h"
 
@@ -545,6 +546,7 @@ enum {
 	INPUT_DATE_FMT,
 	HEADING_POS,
 	DAY_HEADING_FMT,
+	APPOINTMENT_TIME_FMT,
 	NB_OPTIONS
 };
 
@@ -573,7 +575,8 @@ static void print_general_option(int i, WINDOW *win, int y, int hilt, void *cb_d
 		"format.outputdate = ",
 		"format.inputdate = ",
 		"appearance.headingposition = ",
-		"format.dayheading = "
+		"format.dayheading = ",
+		"format.appointmenttime = "
 	};
 	const char *panel;
 	const char *position;
@@ -700,8 +703,7 @@ static void print_general_option(int i, WINDOW *win, int y, int hilt, void *cb_d
 	case FIRST_DAY_OF_WEEK:
 		custom_apply_attr(win, ATTR_HIGHEST);
 		mvwaddstr(win, y, XPOS + strlen(opt[FIRST_DAY_OF_WEEK]),
-			  ui_calendar_week_begins_on_monday()? _("Monday") :
-			  _("Sunday"));
+			  nl_langinfo(DAY_1 + ui_calendar_get_wday_start()));
 		custom_remove_attr(win, ATTR_HIGHEST);
 		mvwaddstr(win, y + 1, XPOS,
 			  _("(specifies the first day of week in the calendar view)"));
@@ -746,6 +748,14 @@ static void print_general_option(int i, WINDOW *win, int y, int hilt, void *cb_d
 		mvwaddstr(win, y + 1, XPOS,
 			  _("(Format of the date displayed in the appointments panel)"));
 		break;
+	case APPOINTMENT_TIME_FMT:
+		custom_apply_attr(win, ATTR_HIGHEST);
+		mvwaddstr(win, y, XPOS + strlen(opt[APPOINTMENT_TIME_FMT]),
+			  conf.timefmt);
+		custom_remove_attr(win, ATTR_HIGHEST);
+		mvwaddstr(win, y + 1, XPOS,
+			  _("(Format of the time displayed in the appointments panel)"));
+		break;
 	}
 
 	if (hilt)
@@ -771,6 +781,8 @@ static void general_option_edit(int i)
 	    _("Enter a text string (an empty string for the default text)");
 	const char *output_datefmt_str =
 	    _("Enter the date format (see 'man 3 strftime' for possible formats) ");
+	const char *output_timefmt_str =
+	    _("Enter the time format (see 'man 3 strftime' for possible formats) ");
 	const char *input_datefmt_prefix = _("Enter the date format: ");
 	const char *periodic_save_str =
 	    _("Enter the delay, in minutes, between automatic saves (0 to disable) ");
@@ -893,6 +905,15 @@ static void general_option_edit(int i)
 			conf.day_heading[BUFSIZ - 1] = '\0';
 		}
 		break;
+	case APPOINTMENT_TIME_FMT:
+		status_mesg(output_timefmt_str, "");
+		strncpy(buf, conf.timefmt, BUFSIZ);
+		buf[BUFSIZ - 1] = '\0';
+		if (updatestring(win[STA].p, &buf, 0, 1) == 0) {
+			strncpy(conf.timefmt, buf, BUFSIZ);
+			conf.timefmt[BUFSIZ - 1] = '\0';
+		}
+		break;
 	}
 
 	mem_free(buf);
@@ -975,10 +996,11 @@ print_keys_bindings(WINDOW * win, int selected_row, int selected_elm,
 	const int XPOS = 1;
 	const int EQUALPOS = 23;
 	const int KEYPOS = 25;
-	int noelm, action, y;
+	int noelm, action, y, pos;
+	const char *key = NULL;
 
 	noelm = y = 0;
-	for (action = 0; action < NBKEYS; action++) {
+	for (action = 0; action < NBVKEYS; action++) {
 		char *actionstr;
 		int nbkeys;
 
@@ -990,18 +1012,15 @@ print_keys_bindings(WINDOW * win, int selected_row, int selected_elm,
 		mem_free(actionstr);
 		mvwaddstr(win, y, EQUALPOS, "=");
 		if (nbkeys == 0)
-			mvwaddstr(win, y, KEYPOS, _("undefined"));
+			mvwaddstr(win, y, KEYPOS, _("UNDEFINED"));
 		if (action == selected_row)
 			custom_remove_attr(win, ATTR_HIGHEST);
 		if (nbkeys > 0) {
 			if (action == selected_row) {
-				const char *key;
-				int pos;
-
+				/* Elements may have been added or deleted. */
+				wclrtoeol(win);
 				pos = KEYPOS;
-				while ((key =
-					keys_action_nkey(action,
-							 noelm)) != NULL) {
+				while ((key = keys_action_nkey(action, noelm))) {
 					if (noelm == selected_elm)
 						print_key_incolor(win, key,
 								  y, pos);
@@ -1012,8 +1031,9 @@ print_keys_bindings(WINDOW * win, int selected_row, int selected_elm,
 					pos += utf8_strwidth((char *)key) + 1;
 				}
 			} else {
-				mvwaddstr(win, y, KEYPOS,
-					  keys_action_allkeys(action));
+				key = keys_action_allkeys(action);
+				mvwaddstr(win, y, KEYPOS, key);
+				mem_free((char *)key);
 			}
 		}
 		y += yoff;
@@ -1045,9 +1065,11 @@ void custom_keys_config(void)
 	const int LABELLINES = 3;
 
 	clear();
-	nbdisplayed = ((notify_bar() ? row - 3 : row - 2) - LABELLINES) / LINESPERKEY;
-	wins_scrollwin_init(&kwin, 0, 0, notify_bar() ? row - 3 : row - 2, col, _("keys configuration"));
-	wins_scrollwin_set_pad(&kwin, NBKEYS * LINESPERKEY);
+	nbdisplayed = ((notify_bar() ? row - 3 : row - 2) -
+		       LABELLINES) / LINESPERKEY;
+	wins_scrollwin_init(&kwin, 0, 0, notify_bar() ? row - 3 : row - 2, col,
+			    _("keys configuration"));
+	wins_scrollwin_set_pad(&kwin, NBVKEYS * LINESPERKEY);
 	wins_scrollwin_draw_deco(&kwin, 0);
 	custom_keys_config_bar();
 	selrow = selelm = 0;
@@ -1072,7 +1094,7 @@ void custom_keys_config(void)
 			}
 			break;
 		case KEY_MOVE_DOWN:
-			if (selrow < NBKEYS - 1) {
+			if (selrow < NBVKEYS - 1) {
 				selrow++;
 				selelm = 0;
 				if (selrow == lastrow) {
@@ -1103,7 +1125,7 @@ void custom_keys_config(void)
 					keys_get_label(selrow), 0);
 			for (;;) {
 				ch = keys_wgetch(grabwin);
-				enum key action = keys_get_action(ch);
+				enum vkey action = keys_get_action(ch);
 				/* Is the key already used by this action? */
 				if (action == selrow)
 					break;
@@ -1142,10 +1164,8 @@ void custom_keys_config(void)
 				selelm--;
 			break;
 		case KEY_GENERIC_QUIT:
-			if (keys_check_missing_bindings() != 0) {
-				WARN_MSG(_("Some actions do not have any associated "
-					  "key bindings!"));
-			}
+			if (keys_check_undefined())
+				WARN_MSG(_("Some actions are left undefined!"));
 			wins_scrollwin_delete(&kwin);
 			return;
 		}
@@ -1222,7 +1242,7 @@ void custom_config_main(void)
 			old_layout = wins_layout();
 			custom_layout_config();
 			if (wins_layout() != old_layout)
-				wins_reset();
+				wins_resize();
 			break;
 		case 'G':
 		case 'g':
@@ -1248,7 +1268,11 @@ void custom_config_main(void)
 			resize = 0;
 			wins_reset();
 		}
-
+		
+		/* needed to update app list */
+		day_do_storage(0);
+		
+		/* wins_update(FLAG_ALL), but with custom bindings */
 		wins_set_bindings(bindings, ARRAY_SIZE(bindings));
 		wins_update_border(FLAG_ALL);
 		wins_update_panels(FLAG_ALL);
