@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2020 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2022 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -241,9 +241,11 @@ enum getstr getstring(WINDOW * win, char *str, int l, int x, int y)
 			st.len = st.pos;
 			break;
 		case CTRL('A'):	/* go to beginning of string */
+		case KEY_HOME:
 			st.pos = 0;
 			break;
 		case CTRL('E'):	/* go to end of string */
+		case KEY_END:
 			st.pos = st.len;
 			break;
 		case KEY_LEFT:	/* move one char backward  */
