@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2020 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2022 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -140,6 +140,7 @@ void vars_init(void)
 	conf.input_datefmt = 1;
 	conf.heading_pos = RIGHT;
 	strcpy(conf.day_heading, DAY_HEADING_DEFAULT);
+	strcpy(conf.timefmt, APPT_TIME_DEFAULT);
 
 	datefmt_str[0] = _("mm/dd/yyyy");
 	datefmt_str[1] = _("dd/mm/yyyy");
@@ -174,6 +175,8 @@ void vars_init(void)
 
 	ui_calendar_set_first_day_of_week(MONDAY);
 
+	wins_set_sbar_width(col * SBARMINWIDTH / 100);
+
 	/* Pad structure to scroll text inside the appointment panel */
 	apad.length = 1;
 	apad.first_onscreen = 0;
